module Utility where

import qualified Views        as V

import Control.Exception      (IOException)
import Control.Monad.Catch    (catch)
import Effectful
import Effectful.Error.Static (Error, throwError)
import Lucid                  (renderBS)
import Servant                ( ServerError(..)
                              , errHTTPCode
                              , errReasonPhrase
                              , errBody
                              , errHeaders
                              )

liftIOE :: (IOE :> es, Error ServerError :> es) => IO a -> Eff es a
-- Lift IO into Eff and catch IOExceptions
liftIOE m = liftIO m `catch` \(e::IOException) -> do
    -- Log IOExceptions to stdout
    liftIO $ putStrLn $ "ERROR Exception: " ++ show e
    -- Throw a custom Servant ServerError
    throwError $ ServerError
      { errHTTPCode     = 500
      , errReasonPhrase = "Internal Server Error"
      , errBody         = renderBS V.internalServerError
      , errHeaders      = []
      }
